#!/usr/bin/env python3

from snake_game import SnakeGame
from functools import reduce
import time
import copy
import json
import gzip

with gzip.open('msp430.snake.qtable.json.gz', 'rt') as f:
  qtable = json.load(f)

def extract_features(game):
  snake, food = game.get_state()
  obsticle = lambda k: copy.deepcopy(game).step(k).done
  o = sum(v << i for i, v in enumerate(obsticle(k) for k in game.ACTION))
  a, b = snake.head.h - food.h, snake.head.w - food.w
  c, d = abs(a) < game.board_height / 2, abs(b) < game.board_width / 2
  a, b = (max(min(1, i), -1) + 1 for i in (a, b))
  c, d = (int(i) for i in (c, d))
  return o, a, b, c, d

def extract_features_wrapper(game):
  win, gui = game.win, game.gui
  game.win, game.gui = None, False
  fe = extract_features(game)
  game.win, game.gui = win, gui
  return fe

try:
  game = SnakeGame(gui = True)
  while not game.done:
    time.sleep(0.2)
    fe = extract_features_wrapper(game)
    action = reduce(lambda a, i: a[i], fe, qtable)
    state, *done_reward, info = game.step(action)

except KeyboardInterrupt:
  pass

game.end_game()
*state, info = (tuple(p) for p in (
    state.snake.head, *state.snake.body, state.food, info) if p)
print(*state, *done_reward, info)